"use strict";

var nocardString = 'nocard';
var accessoryCardsAPIurl = 'http://127.0.0.1/api/system/accessoryCards';
var builtInLoraCardsAPIurl = 'http://127.0.0.1/api?fields=system/capabilities/lora,system/loraProductId';
var http = require('http');

// Returns a string with the product ID in the specified port or the nocardString
exports.getProductId = function (port, callback) {
  http.get(accessoryCardsAPIurl, function(result) {
    result.on('data', function(data) {
      // console.log(data);
      var accessoryCards = JSON.parse(data);

      if (accessoryCards.result[port - 1]) {
          // console.log('mts-mcard: ' + accessoryCards.result[port - 1].productId);
          callback(accessoryCards.result[port - 1].productId);
      }
      else {
          // console.error('mts-mcard: No accessory card in port ' + port + '!');
          callback(nocardString);
      }
    });
  }).on('error', function(e) {
    console.error('mts-mcard: ' + e);
    callback('error');
  });
};

// Returns an array with the present product IDs or the nocardString if a port is empty
exports.getProductIds = function (callback) {
  http.get(accessoryCardsAPIurl, function(result) {
    result.on('data', function(data) {
      // console.log(data);
      var accessoryCards = JSON.parse(data);
      var productIds = [nocardString, nocardString];

      if (accessoryCards.result[0]) {
          productIds[0] = accessoryCards.result[0].productId;
      }
      if (accessoryCards.result[1]) {
          productIds[1] = accessoryCards.result[1].productId;
      }
      callback(productIds);
    });
  }).on('error', function(e) {
    console.error('mts-mcard: ' + e);
    callback('error');
  });
};

// Accepts an array of valid ids.
// Returns true if an mCard is found with one of those ID's, false otherwise
exports.compatableProductIdMatches = function (idRegex, callback) {
  http.get(accessoryCardsAPIurl, function(result) {
    result.on('data', function(data) {
      // console.log(data);
      var accessoryCards = JSON.parse(data);

      if (accessoryCards.result[0]) {
        if (idRegex.test(accessoryCards.result[0].productId)) {
          callback(true);
          return;
        }
      }

      if (accessoryCards.result[1]) {
        if (idRegex.test(accessoryCards.result[1].productId)) {
          callback(true);
          return;
        }
      }

      callback(false);
    });
  }).on('error', function(e) {
    console.error('mts-mcard: ' + e);
    callback('error');
  });
};

// Accepts an array of valid ids.
// Returns true if an mCard is found with one of those ID's, false otherwise
exports.compatableProductIdPresent = function (compatableMcardIDs, callback) {
  http.get(accessoryCardsAPIurl, function(result) {
    result.on('data', function(data) {
      // console.log(data);
      var accessoryCards = JSON.parse(data);

      if (accessoryCards.result[0]) {
        if (compatableMcardIDs.indexOf(accessoryCards.result[0].productId) != -1) {
          callback(true);
          return;
        }
      }

      if (accessoryCards.result[1]) {
        if (compatableMcardIDs.indexOf(accessoryCards.result[1].productId) != -1) {
          callback(true);
          return;
        }
      }

      callback(false);
    });
  }).on('error', function(e) {
    console.error('mts-mcard: ' + e);
    callback('error');
  });
};

// Accepts an array of valid ids.
// Returns true if an built-in Lora Card is found with one of those ID's, false otherwise
exports.compatableBuiltInLoraProductIdPresent = function (node, ids, callback) {
  var http = require('http');
  http.get(builtInLoraCardsAPIurl, function(result) {
    result.on('data', function(data) {
      var response = JSON.parse(data);
      if (response.hasOwnProperty("result") &&
          response.result.hasOwnProperty("system_capabilities_lora") &&
          response.result.hasOwnProperty("system_loraProductId")) {

          var lora_capa = response.result.system_capabilities_lora;
          var lora_productid  = response.result.system_loraProductId;
          if (typeof lora_capa == "boolean" && lora_capa &&
              typeof lora_productid == "string" && lora_productid.length > 0) {
              if (ids.indexOf(lora_productid) != -1) {
                callback(true);
                return;
              }
          }
      }
      callback(false);
    });
  }).on('error', function(e) {
    console.error('mts-mcard lora: ' + e);
    callback('error');
  });
};

// Accepts an array of valid ids.
// Returns true if an built-in Lora Card is found with one of those ID's, false otherwise
exports.compatableBuiltInLoraProductIdMatches = function (node, idRegex, callback) {
  var http = require('http');
  http.get(builtInLoraCardsAPIurl, function(result) {
    result.on('data', function(data) {
      var response = JSON.parse(data);
      if (response.hasOwnProperty("result") &&
          response.result.hasOwnProperty("system_capabilities_lora") &&
          response.result.hasOwnProperty("system_loraProductId")) {

          var lora_capa = response.result.system_capabilities_lora;
          var lora_productid  = response.result.system_loraProductId;
          if (typeof lora_capa == "boolean" && lora_capa &&
              typeof lora_productid == "string" && lora_productid.length > 0) {
              if (idRegex.test(lora_productid)) {
                  callback(true);
                  return;
              }
          }
      }
      callback(false);
    });
  }).on('error', function(e) {
    console.error('mts-mcard lora: ' + e);
    callback('error');
  });
};


