module.exports = function(RED) {

  "use strict";
  
  var mqtt = require('mqtt');
  var mqttServer = "mqtt://127.0.0.1'";
  var Mcard = require('mts-mcard');

  function setStatus(node, status) {
    switch (status) {
      case "connecting": node.status({fill:"grey",shape:"dot",text:"connecting"}); break;
      case "connected": node.status({fill:"green",shape:"dot",text:"connected"}); break;
      case "disconnected": node.status({fill:"red",shape:"ring",text:"disconnected"}); break;
      case "error": node.status({fill:"red",shape:"ring",text:"ERROR-MQTT CONNECT"}); break;
      default: node.status({}); break;
    }
  }


  function LoraInNode(config) {
    RED.nodes.createNode(this, config);
    var node = this;

    setStatus(node, "connecting");   
    SetupLoraInNode(node, config);

    node.log("lora in node created");
  }

  function SetupLoraInNode(node, config) {
    var mqttTopic = 'lora/+/up';
    var client = mqtt.connect(mqttServer).on('error', function(error) {
      node.error("LoRa in node MQTT error: " + error);
      setStatus(node, "error");
    }).on('connect', function() {
      node.log("Connected to MQTT for LoRa in node.");
      setStatus(node, "connected");
      client.subscribe(mqttTopic);
    }).on("reconnect", function() {
      node.log("LoRa-IN node reconnect");
      setStatus(node, "disconnected");
    }).on("error", function() {
      node.log("LoRa-IN node error");
      setStatus(node, "disconnected");
    }).on('message', function(topic, message) {
      var arr = topic.split("/");
      var obj = JSON.parse(message);
      var data;
      if (config.datatype === "utf8") {
        data = new Buffer(obj.data, 'base64').toString('utf-8');
      } else if (config.datatype === "bytes") {
        data = new Buffer(obj.data, 'base64');
      } else {
        node.error("Invalid datatype.");
        return;
      }
      if (obj.data) { // moving the data to the payload for use in Node-RED
        obj.payload = data;
        delete obj.data;
      }
      obj.eui = arr[1];
      node.send(obj);
    });
            
    node.on("close", function() {
      node.log("LoRa in node closing...");
      setStatus(node, "disconnected");
      client.end();
    });
  }

  RED.nodes.registerType("lora in", LoraInNode);

  function LoraOutNode(config) {
    RED.nodes.createNode(this, config);
    var node = this;

    setStatus(node, "connecting");
    SetupLoraOutNode(node, config);

    node.log("lora out node created");
  }

  function SetupLoraOutNode(node, config) {
    var client = mqtt.connect(mqttServer).on('error', function(error) {
      node.error("LoRa out node MQTT error: " + error);
      setStatus(node, "error");
    }).on('connect', function() {
      node.log("Connected to MQTT for LoRa out node.");
      setStatus(node, "connected");
    }).on("reconnect", function() {
      node.log("LoRa-OUT node reconnect");
      setStatus(node, "disconnected");
    }).on("error", function() {
      node.log("LoRa-OUT node error");
      setStatus(node, "disconnected");
    });
    
    node.on('input', function(msg) {
      var eui = msg.eui || config.eui;
      var payload = msg.payload || config.payload;
      var ack_config = (msg.ack || config.ack) == true;
      var port_config = parseInt(msg.port || config.port, 10);
      
      if (payload instanceof Buffer || typeof(payload) === "string" ) {
        var msg = { data: new Buffer(payload).toString("base64"), ack: ack_config, port: port_config };
        client.publish('lora/' + eui + '/down', JSON.stringify(msg));
        node.log("Published downstream lora packet");
      } else {
        node.error("Invalid data type in message payload: " + typeof payload + ".  Expected String or Buffer.");
        return;
      }

    }); 
    
    node.on("close", function() {
      node.log("LoRa out node closing...");
      setStatus(node, "disconnected");
      client.end();
    });
  }

  RED.nodes.registerType("lora out", LoraOutNode);

};
